/* eslint-disable jsx-a11y/click-events-have-key-events */
/* eslint-disable jsx-a11y/no-static-element-interactions */
/* eslint-disable @wordpress/i18n-text-domain */
/* eslint-disable @typescript-eslint/no-use-before-define */
/**
 * External dependencies
 */
import { sprintf, __ } from '@wordpress/i18n';
import {
	Fragment,
	useState,
	createRoot,
	useRef,
	useEffect,
} from '@wordpress/element';
import domReady from '@wordpress/dom-ready';

/**
 * Internal dependencies
 */
import Checked from './components/checked';
import Message from './components/message';
import Undo from './components/undo';
import ArrowLeft from './components/arrow-left';
import Share from './components/share';
import './index.scss';

// 图片资源
const images = {
	logo: 'https://assets.staticmeow.com/customeow/static/open-component-cover/2024126/d5190418-0928-46b7-8f9d-6518085e38cc.webp',
	guide: 'https://assets.staticmeow.com/customeow/static/open-component-cover/2024126/e891c6f0-9844-414c-84dd-517e0ae345b1.jpeg',
	connected:
		'https://assets.staticmeow.com/customeow/static/open-component-cover/2024126/8a179faa-1434-4dec-8bd5-2cf03164ce08.webp',
	articleDownload:
		'https://assets.staticmeow.com/customeow/static/editor-image/2024126/m4ce6fol.webp',
	articleIntegrate:
		'https://assets.staticmeow.com/customeow/static/editor-image/2024124/m49ckjls.webp',
	articleSupport:
		'https://assets.staticmeow.com/customeow/static/open-component-cover/2024126/147b8f2c-2fcb-4a41-8ff3-2ef0e9da5aaa.webp',
};

const CustoMeowAdminPage = () => {
	const { customeow_admin_data = undefined } = window;

	const [ showMessage, setShowMessage ] = useState( { value: false } );
	const [ isConnect, setIsConnect ] = useState(
		customeow_admin_data?.connect_status === '0' ? false : true
	);
	const [ storeUrl, setStoreUrl ] = useState(
		customeow_admin_data?.store_url || 'https://customeow.io'
	);
	const [ isPolling, setIsPolling ] = useState( false );
	const pollingRef = useRef( null );
	const startTimeRef = useRef( null );

	useEffect( () => {
		if ( ! customeow_admin_data ) return;

		if ( customeow_admin_data.connect_status === '0' && ! isPolling ) {
			setIsPolling( true );
			startTimeRef.current = new Date().getTime();
			poll();
		}

		return () => {
			clearTimeout( pollingRef.current );
		};

		// eslint-disable-next-line react-hooks/exhaustive-deps
	}, [] );

	const poll = () => {
		if ( ! customeow_admin_data ) return;

		const { ajax_url, security } = customeow_admin_data;

		window.jQuery.ajax( {
			url: ajax_url,
			method: 'POST',
			data: {
				action: 'ajax_force_check_customeow_connect_status',
				security,
			},
			success: ( response ) => {
				if ( response.data.is_connect ) {
					clearTimeout( pollingRef.current );
					setIsConnect( true );
					setStoreUrl( response.data.store_url );
					setIsPolling( false );
				} else {
					checkPollingTime();
				}
			},
			error: () => {
				checkPollingTime();
			},
		} );
	};

	const checkPollingTime = () => {
		const currentTime = new Date().getTime();
		if ( currentTime - startTimeRef.current >= 10 * 60 * 1000 ) {
			clearTimeout( pollingRef.current );
			setIsPolling( false );
		} else {
			pollingRef.current = setTimeout( poll, 10000 );
		}
	};

	const unsecuredCopyToClipboard = ( text ) => {
		const textArea = document.createElement( 'textarea' );
		textArea.style.position = 'fixed';
		textArea.style.top = 0;
		textArea.style.left = 0;
		textArea.style.opacity = 0;

		textArea.value = text;
		document.body.appendChild( textArea );
		textArea.focus();
		textArea.select();
		try {
			document.execCommand( 'copy' );
			setShowMessage( { value: true } );
		} catch ( err ) {
			const msg = __( 'Unable to copy to clipboard', 'woocommerce' );
			console.error( msg, err );
		}
		document.body.removeChild( textArea );
	};

	const handleCopy = async () => {
		if ( window.isSecureContext && window.navigator.clipboard ) {
			window.navigator.clipboard.writeText(
				customeow_admin_data?.home_url
			);
			setShowMessage( { value: true } );
		} else {
			unsecuredCopyToClipboard( customeow_admin_data?.home_url );
		}
	};

	const handlerConnect = () => {
		if ( customeow_admin_data ) {
			const connect_url = customeow_admin_data.connect_url;
			window.open( connect_url );
		}
	};

	// if ( customeow_admin_data && customeow_admin_data.issues.length !== 0 ) {
	// 	return (
	// 		<>
	// 			<Error errors={ customeow_admin_data?.issues ?? [] } />
	// 		</>
	// 	);
	// }

	return (
		<Fragment>
			<Message
				show={ showMessage }
				message={ __(
					'🎉 Copied successfully！！！',
					'customeow-product-personalizer'
				) }
			/>

			<div className="customeow-contanier">
				<div className="customeow-plugin-logo customeow-plugin-transition">
					<a href={ customeow_admin_data?.customeow_url }>
						<img src={ images.logo } alt="CustoMeow" />
					</a>
				</div>

				{ /* install plugin  */ }
				<div className="customeow-plugin-container customeow-plugin-flex">
					<span className="customeow-plugin-icon">
						<Checked className="customeow-plugin-icon-success" />
					</span>
					<div className="customeow-plugin-content">
						<h1 className="customeow-plugin-h1">
							{ __(
								'Install CustoMeow plugin',
								'customeow-product-personalizer'
							) }
						</h1>
						<p className="customeow-plugin-p">
							{ __(
								'CustoMeow plugin installed successfully!',
								'customeow-product-personalizer'
							) }
						</p>
						<div className="customeow-plugin-tag-group">
							<span className="customeow-plugin-version">
								{ __(
									'Version',
									'customeow-product-personalizer'
								) }{ ' ' }
								{ customeow_admin_data?.plugin_version }
							</span>
						</div>
					</div>
				</div>

				{ /*  unconnect to customeow  */ }
				<div className="customeow-plugin-container customeow-plugin-flex">
					<span className="customeow-plugin-icon">
						{ isConnect ? (
							<Checked className="customeow-plugin-icon-success" />
						) : (
							<Undo className="customeow-plugin-icon-warning" />
						) }
					</span>
					<div
						className="customeow-plugin-content"
						style={ { display: 'flex' } }
					>
						<div style={ { flex: 1, marginRight: '20px' } }>
							<h1 className="customeow-plugin-h1">
								{ __(
									'Connect WooCommerce to CustoMeow',
									'customeow-product-personalizer'
								) }
							</h1>
							<p className="customeow-plugin-p">
								{ isConnect
									? __(
											'You can manage custom features and design templates in CustoMeow, and also view complete customization data for your orders.',
											'customeow-product-personalizer'
									  )
									: __(
											'Sign in to CustoMeow, go to My Stores and connect your WooCommerce store.',
											'customeow-product-personalizer'
									  ) }
							</p>
							<div className="customeow-plugin-box">
								{ isConnect ? (
									<a
										href={ storeUrl }
										target="_blank"
										className="customeow-button-primary"
										rel="noreferrer"
									>
										{ __(
											'Go to CustoMeow',
											'customeow-product-personalizer'
										) }
										<span style={ { marginLeft: '4px' } }>
											<Share />
										</span>
									</a>
								) : (
									<div
										onClick={ handlerConnect }
										className="customeow-button-primary"
									>
										{ __(
											'Connect Now',
											'customeow-product-personalizer'
										) }
										<span style={ { marginLeft: '4px' } }>
											<ArrowLeft />
										</span>
									</div>
								) }
							</div>
						</div>
						<div style={ { width: '360px' } }>
							<img
								style={ {
									borderRadius: '8px',
									display: 'flex',
									width: '100%',
								} }
								src={
									isConnect ? images.connected : images.guide
								}
								alt="Connect to CustoMeow"
							/>
						</div>
					</div>
				</div>

				{ /*  recommend  */ }
				<div className="customeow-recommend">
					{ /*  check new version */ }
					<a
						href="https://customeow.io/pages/woocommerce-plugin"
						target="_blank"
						className="customeow-article-card"
						rel="noreferrer"
					>
						<img
							src={ images.articleDownload }
							alt="Check & Download Plugin"
						/>
						<p>
							{ __(
								'Check & Download Plugin',
								'customeow-product-personalizer'
							) }
						</p>
					</a>

					{ /* integrate */ }
					<a
						href="https://customeow.io/integrations/woocommerce"
						target="_blank"
						className="customeow-article-card"
						rel="noreferrer"
					>
						<img
							src={ images.articleIntegrate }
							alt="Best Product Personalizer For WooCommerce"
						/>
						<p>
							{ __(
								'Best Product Personalizer For WooCommerce',
								'customeow-product-personalizer'
							) }
						</p>
					</a>

					{ /* support  */ }
					<a
						href="mailto://support@customeow.io"
						className="customeow-article-card"
					>
						<img
							src={ images.articleSupport }
							alt="Contact Support"
						/>
						<p>
							{ __(
								'Contact Support',
								'customeow-product-personalizer'
							) }
						</p>
					</a>
				</div>
			</div>
		</Fragment>
	);
};

domReady( () => {
	const customeowText =
		'' +
		'     ____                   __                                                 ' +
		'\n' +
		"    /\\  _`\\                /\\ \\__         /'\\_/`\\                              " +
		'\n' +
		'    \\ \\ \\/\\_\\  __  __   ___\\ \\ ,_\\   ___ /\\      \\     __    ___   __  __  __  ' +
		'\n' +
		"     \\ \\ \\/_/_/\\ \\/\\ \\ /',__\\ \\ \\/  / __`\\ \\ \\__\\ \\  /'__`\\ / __`\\/\\ \\/\\ \\/\\ \\ " +
		'\n' +
		'      \\ \\ \\L\\ \\ \\ \\_\\ /\\__, `\\ \\ \\_/\\ \\L\\ \\ \\ \\_/\\ \\/\\  __//\\ \\L\\ \\ \\ \\_/ \\_/ \\ ' +
		'\n' +
		"       \\ \\____/\\ \\____\\/\\____/\\ \\__\\ \\____/\\ \\_\\\\ \\_\\ \\____\\ \\____/\\ \\___x___/' " +
		'\n' +
		'        \\/___/  \\/___/ \\/___/  \\/__/\\/___/  \\/_/ \\/_/\\/____/\\/___/  \\/__//__/  ' +
		'\n' +
		'';

	console.log(
		customeowText +
			'\n\n' +
			'        Build custom products better.\n\n' +
			'        developer@customeow.io\n\n'
	);

	const rootElement = document.getElementById( 'customeow_pp_admin_root' );
	if ( rootElement ) {
		const root = createRoot( rootElement );

		root.render( <CustoMeowAdminPage /> );
	}
} );
