/**
 * External dependencies
 */
import { useState, useEffect, useRef } from '@wordpress/element';

/**
 * Internal dependencies
 */
import defaultConfig from '../../utils/default-config';

import './index.scss';

const CustomeowWrapper = ( {
	ajax_url,
	// cart_url,
	security,
	shop_config,
	response,
	variation_id,
	product_id,
	is_no_variation_product,
} ) => {
	/** json 数据 */
	const [ internalProps, setInternalProps ] = useState( response );
	/** 当前变体 */
	const [ variationId, setVariationId ] = useState( variation_id );
	/** 当前数量 */
	const [ quantity, setQuantity ] = useState( 1 );
	/** 遮罩Dom数组 */
	const _maskDoms = useRef( [] );
	/** 来源按钮元素Dom数组  */
	const _sourceDoms = useRef( [] );
	/** 观察者 */
	const _observer = useRef( null );
	/** 定时器 */
	const _timer = useRef( null );
	/** 表单 */
	const _form = useRef( document.querySelector( 'form.cart' ) );
	/** 是否显示遮罩 */
	const _isShowMaskDom = useRef( true );
	/** jQuery */
	const { jQuery } = window;
	/** 默认加车按钮类名 */
	const defaultButtonClassName = '.single_add_to_cart_button';
	/** 加载中按钮类名 */
	const loadingClassName = 'customeow-app-skeleton-loading';
	/** 需要遮罩的按钮 */
	const buttonGroup = shop_config?.buttonGroup || {};
	// 缓存变体json
	window._cacheVariationJson = {};

	/** 创建一个撑满父级容器的元素 */
	const createFullDiv = () => {
		return jQuery( '<div></div>' )
			.attr( {
				'data-type': 'customeowApp',
			} )
			.css( {
				display: _isShowMaskDom.current ? 'block' : 'none',
				cursor: 'pointer',
				position: 'absolute',
				top: 0,
				left: 0,
				width: '100%',
				height: '100%',
				'z-index': 1999999997,
			} );
	};

	/** 处理DOM position属性 */
	const handleDomPositionProperty = ( element ) => {
		// 判断元素定位 - 如果为static或none，则设置relative定位
		const position = element.css( 'position' );
		if ( position === 'static' ) element.css( 'position', 'relative' );
	};

	/** 处理按钮类名 */
	const handleButtonGroup = () => {
		if ( ! Array.isArray( buttonGroup[ 'addToCart' ] ) ) {
			buttonGroup[ 'addToCart' ] = [ defaultButtonClassName ];
		}
	};

	/** 上报错误 */
	const abnormalReport = () => {
		jQuery.ajax( {
			url: ajax_url,
			method: 'POST',
			data: {
				action: 'abnormal_report_from_customeow',
				platform_id: internalProps.platformId,
				security,
			},
			success() {},
			error( error ) {
				console.log( 'abnormalReport error:', error );
			},
		} );
	};

	/** 添加遮罩 */
	const addMask = ( className ) => {
		const targetEle = jQuery( className );
		if ( targetEle.length ) {
			if ( _sourceDoms.current.find( ( dom ) => dom.is( targetEle ) ) )
				return;

			const fullDiv = createFullDiv();
			handleDomPositionProperty( targetEle );
			fullDiv.on( 'click', ( event ) => {
				// 阻止事件冒泡
				event.stopPropagation();
				event.preventDefault();
				if ( fullDiv.hasClass( loadingClassName ) ) return;
				// 触发设计弹窗
				jQuery( '#customeow_pp_button_root' )
					.find( '.__button' )
					.trigger( 'click' );
			} );

			targetEle.append( fullDiv );
			_maskDoms.current.push( fullDiv );
			_sourceDoms.current.push( targetEle );
		} else {
			return className;
		}
	};

	/** 配置按钮添加遮罩 */
	const buttonsAddMask = () => {
		clearTimeout( _timer.current );
		// 断开观察者
		if ( _observer.current ) {
			_observer.current.disconnect();
		}
		// 失败按钮组
		const failButtonGroup = { addToCart: [], buyItNow: [] };

		if ( buttonGroup ) {
			Object.keys( buttonGroup ).forEach( ( key ) => {
				buttonGroup[ key ]?.forEach( ( className ) => {
					try {
						const result = addMask( className );
						if ( result ) failButtonGroup[ key ].push( result );
					} catch ( error ) {
						failButtonGroup[ key ].push( className );
						console.log( 'add mask error:', error );
					}
				} );
			} );
		}

		if ( _maskDoms.current.length === 0 ) {
			// 保底加车按钮
			addMask( defaultButtonClassName );
		}

		if ( _maskDoms.current.length === 0 ) {
			// 如果加车按钮都识别失败，则上报错误
			abnormalReport();
		}

		// 如果观察者为空，则监听表单变化
		if ( _observer.current === null ) {
			watchFormChange();
		}
	};

	/** 透明遮罩元素集合显隐 */
	const maskShowHide = ( show ) => {
		_isShowMaskDom.current = show;
		_maskDoms.current.forEach( ( element ) => {
			element.css( 'display', show ? 'block' : 'none' );
		} );
		// 替换文案
		handleReplaceText( ! show );
	};

	/** loding 显隐 */
	const loadingVisible = ( visibel ) => {
		if ( visibel ) {
			// 显示
			_maskDoms.current.forEach( ( element ) => {
				element.addClass( loadingClassName );
			} );
		} else {
			// 隐藏
			_maskDoms.current.forEach( ( element ) => {
				element.removeClass( loadingClassName );
			} );
		}
	};

	/** 处理变体改变 */
	const handlerChange = function () {
		const _variationId = window.jQuery( this ).val();
		maskShowHide( true );

		if ( _variationId.length === 0 ) {
			maskShowHide( false );

			setInternalProps( ( preProps ) => ( {
				...preProps,
				available: false,
			} ) );

			return;
		}

		setVariationId( _variationId );

		// 获取缓存变体json
		const currentVariationJson = window._cacheVariationJson[ _variationId ];
		if ( currentVariationJson ) {
			setInternalProps( currentVariationJson );
			if ( currentVariationJson.available ) {
				maskShowHide( true );
			} else {
				maskShowHide( false );
			}

			return;
		}

		loadingVisible( true );
		window.jQuery.ajax( {
			url: ajax_url,
			method: 'POST',
			data: {
				action: 'change_variation_from_customeow',
				variation_id: _variationId,
				product_id,
				security,
			},
			success( result ) {
				if ( result.success && result.data.available ) {
					maskShowHide( true );
				} else {
					maskShowHide( false );
				}

				// 缓存变体json
				window._cacheVariationJson[ _variationId ] = result.data;
				setInternalProps( result.data );
			},
			complete() {
				loadingVisible( false );
			},
		} );
	};

	/** 根据类型生成加车参数 */
	const generateAddToCartParamsByType = ( customInfoId, preview, stores ) => {
		// 隐藏定制数据类型: 0=显示全部，1=显示预览，2=隐藏预览，3=全部隐藏
		const hideCustomDataType = shop_config.hideCustomData;
		// 全部隐藏
		if ( hideCustomDataType === 3 ) return [];

		const customId = customInfoId
			? [ { propertyName: 'customInfoId', value: customInfoId } ]
			: [];

		// 生成preview加车参数
		const paramsPreview = Object.entries( preview )
			// eslint-disable-next-line @typescript-eslint/no-unused-vars
			.filter( ( [ _, value ] ) => value && value.length > 0 )
			.map( ( [ key, value ] ) => ( {
				propertyName: 'preview',
				key,
				value,
			} ) );

		// 判断是否显示全部
		if ( hideCustomDataType === 0 ) {
			const params = customId.concat( paramsPreview ).concat( stores );

			// 处理value为数组的情况
			params.forEach( ( item ) => {
				if ( Array.isArray( item.value ) ) {
					item.value = item.value.map( ( childValue ) => {
						return Array.isArray( childValue )
							? childValue.join( ', ' )
							: childValue;
					} );
				}
			} );
			return params;
		}

		// 只显示预览
		return paramsPreview;
	};

	/** 确认加车处理 */
	const handleConfirm = ( customInfoId, preview, stores, upsellProduct ) => {
		let upsells = [];
		// 如果存在upsellProduct，则添加到加车参数中
		if ( upsellProduct && upsellProduct.length > 0 ) {
			upsells = upsellProduct
				.filter( ( p ) => p )
				.map( ( product ) => ( {
					product_id: product.productId,
					quantity: 1,
					variation_id: product.variantId || 0,
				} ) );
		}

		// 生成c加车参数
		const properties = generateAddToCartParamsByType(
			customInfoId,
			preview,
			stores
		);
		// console.log( '🛒 cart properties:', properties );

		// 获取变体标题
		const productVariations =
			jQuery( '.variations_form' ).data( 'product_variations' ) || [];
		const currentVariation = productVariations.find(
			( i ) => i.variation_id === Number( variationId )
		);
		let variantTitle = '';
		if ( currentVariation ) {
			variantTitle = Object.values( currentVariation?.attributes )?.join(
				'/'
			);
		}

		try {
			// 初始化mini购物车
			jQuery( document.body ).trigger( 'added_to_cart' );
		} catch ( error ) {
			console.log( '🛒 cart error:', error );
		}

		jQuery.ajax( {
			url: ajax_url,
			method: 'POST',
			data: {
				action: 'add_to_woocommerce_cart_from_customeow',
				variation_id: variationId,
				variant_title: variantTitle,
				custom_info_id: customInfoId,
				product_id,
				quantity,
				security,
				properties,
				upsells,
			},
			success( res ) {
				jQuery( document.body ).trigger( 'added_to_cart' );
				jQuery( '.woocommerce-notices-wrapper' )
					.empty()
					.append( res.data.notices );
			},
			error( error ) {
				console.log( '🛒 add cart error:', error );
			},
		} );
	};

	/** 查找首个文本节点，并替换文本节点内容 */
	const replaceFirstTextContent = (
		element,
		replacementText,
		originElement
	) => {
		// 判断element元素是否为自闭合标签
		const selfClosingTags = [ 'img', 'input' ];
		if (
			selfClosingTags.includes( element.prop( 'tagName' ).toLowerCase() )
		) {
			return false;
		}

		// 首先检查自身的子节点中是否有文本节点
		let found = false;
		element.contents().each( ( _, node ) => {
			if (
				node.nodeType === 3 && // 3 表示文本节点
				jQuery.trim( node.nodeValue ) !== ''
			) {
				// 将原始值保存在originElement originalText属性中(已存在则忽略)
				if ( ! originElement.data( 'originalText' ) ) {
					originElement.data( 'originalText', node.nodeValue );
				}
				node.nodeValue = replacementText; // 替换文本内容
				found = true;
				return false; // 找到并替换后跳出循环
			}
		} );

		if ( found ) {
			return true;
		}

		// 如果自身没有文本节点，则递归查找子元素中的文本节点
		let replaced = false;
		element.children().each( ( _, child ) => {
			const childElement = jQuery( child );
			replaced = replaceFirstTextContent(
				childElement,
				replacementText,
				originElement
			);
			if ( replaced ) {
				return false; // 找到并替换后跳出循环
			}
		} );

		return replaced;
	};

	/** 处理替换文案功能 */
	const handleReplaceText = ( shouldRestoreOriginal ) => {
		const { enableCustomText, customText, i18nCustomText, language } =
			shop_config;
		let replaceText = i18nCustomText?.[ language ] || customText;
		if ( ! enableCustomText || ! customText ) return;

		if ( language === 'en_US' ) {
			replaceText = replaceText.toUpperCase();
		}

		const sourceDoms = _sourceDoms.current;
		sourceDoms.forEach( ( dom ) => {
			const originalText = dom.data( 'originalText' );
			const textToApply = shouldRestoreOriginal
				? originalText
				: replaceText;

			// 只在需要替换时执行
			if ( shouldRestoreOriginal && ! originalText ) return;
			replaceFirstTextContent( dom, textToApply, dom );
		} );
	};

	/** 监听表单变化 */
	const watchFormChange = () => {
		// eslint-disable-next-line no-undef
		_observer.current = new MutationObserver( ( mutationsList ) => {
			for ( const mutation of mutationsList ) {
				if (
					mutation.type === 'childList' &&
					mutation.addedNodes.length > 0
				) {
					clearTimeout( _timer.current );
					_timer.current = setTimeout( () => {
						// 清空按钮并重新选择
						_maskDoms.current.forEach( ( dom ) => dom.remove() );
						_maskDoms.current = [];
						_sourceDoms.current = [];
						buttonsAddMask();
						handleReplaceText();
					}, 400 );
				}
			}
		} );
		// 配置 MutationObserver 监听子节点变化
		_observer.current.observe( _form.current, {
			childList: true,
			subtree: true,
		} );
	};

	useEffect( () => {
		// 缓存变体json
		if ( internalProps.customData || internalProps.upsells )
			window._cacheVariationJson = { [ variationId ]: internalProps };
		// 处理按钮类名
		handleButtonGroup();
		// 添加遮罩
		buttonsAddMask();
		// 处理替换文案
		handleReplaceText( false );

		const variationInput = jQuery( 'input[name="variation_id"]' );

		variationInput.on( 'change', handlerChange );

		const handlerQty = function () {
			setQuantity( this.value );
		};

		jQuery( 'input[name="quantity"]' ).on( 'change', handlerQty );

		// 当前变体id
		const currentVariationId = variationInput.val();

		const isHasVariationId =
			currentVariationId && currentVariationId !== '0';

		const isNoCustomData =
			! internalProps?.customData && ! internalProps?.upsells;

		if ( ! internalProps?.available || isNoCustomData ) {
			maskShowHide( false );
		} else if ( ! is_no_variation_product && ! isHasVariationId ) {
			maskShowHide( false );
		}

		return () => {
			jQuery( 'input[name="variation_id"]' ).off(
				'change',
				handlerChange
			);

			jQuery( 'input[name="quantity"]' ).off( 'change', handlerQty );
		};
	}, [] );

	return (
		<Customeow
			newProps={ internalProps }
			shop={ shop_config || defaultConfig }
			productId={ product_id }
			onConfirm={ handleConfirm }
		/>
	);
};

const Customeow = ( { newProps, shop, productId, onConfirm } ) => {
	useEffect( () => {
		const props = Object.assign( {}, newProps, {
			env: shop.env === 'production' ? 'prod' : shop.env || 'dev',
			platformProductId: productId,
			shop,
			productId,
			onConfirm,
		} );

		if ( props.customData || props.upsells )
			window.CustomeowApp.render(
				props,
				document.getElementById( 'customeow_pp_button_root' )
			);
	}, [ newProps, shop, productId, onConfirm ] );

	return null;
};

export default CustomeowWrapper;
